using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.ComponentModel.Design;

namespace MySmeter
{
    [DesignTimeVisible(true)]
    [Category("Components")]
    [Description("Smeter for radio receivers")]
    public partial class MySmeter : UserControl
    {

        //
        // --------------------------------------------- Smeter30
        //private float _zeroAngle = 0;         // angle in degree for the zero position
        //private float _dbAngle = 1.4f;        // angle in degree of 1 decibel span
        //private float _maxDecibels = 35;
        //private float _minDecibels = -35;
        // --------------------------------------------- Smeter40
        //private float _zeroAngle = -3;         // angle in degree for the zero position
        //private float _dbAngle = 1.28f;        // angle in degree of 1 decibel span
        //private float _maxDecibels = 41;
        //private float _minDecibels = -36;
        // --------------------------------------------- Smeter
        private float _zeroAngle = 8;         // angle in degree for the zero position
        private float _dbAngle = 1.00f;        // angle in degree of 1 decibel span
        private float _maxDecibels = 41;
        private float _minDecibels = -57;
        // --------------------------------------------- 
        private double _oldAngle = -0.8;
        private float _speed = 0.1f;
        private float _centerX = 0.485f;       // rotation center X as a rapport of the width
        private float _centerY = 1.6f;         // rotation center Y as a rapport of the height
        private float _lineLen = 1.45f;        // line length as a rapport of the height
        private float _lineLenPixels;          // line length in pixels
        // --------------------------------------------- 
        private Pen _armPen = new Pen(Color.FromArgb(90,80,60), 3);
        private Brush _transparentBrush = new SolidBrush(Color.FromArgb(150, 200, 200, 210));
        private Brush _textBrush = new SolidBrush(Color.FromArgb(30, 100, 0));

        public float _trimmingDecibels;
        public double _trimHi;
        public double _trimLow;
        private Graphics g;
        private Bitmap _originalBackBitmap;
        private Bitmap _backBitmap;
        private double krad = 2 * Math.PI / 360;
        private int w, h;
        private float x0, y0;
        private double peakVeff;
        private Stopwatch sw = new Stopwatch();

        public MySmeter()
        {
            InitializeComponent();
            _originalBackBitmap = (Bitmap)BackgroundImage.Clone();
            sw.Reset();
            sw.Start();
        }

        private void MySmeter_Resize(object sender, EventArgs e)
        {
            w = ClientSize.Width;
            h = ClientSize.Height;
            _backBitmap = new Bitmap(w, h);
            g = Graphics.FromImage(_backBitmap);
            g.DrawImage(_originalBackBitmap, 0, 0, w, h);
            BackgroundImage = (Bitmap)_backBitmap.Clone();
            g = Graphics.FromImage(BackgroundImage);
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            x0 = w * _centerX;
            y0 = h * _centerY;
            _lineLenPixels = _lineLen * h;
            Update(-0.8);
        }

        public double TrimHi
        {
            set { _trimHi = Math.Pow(10, value / 20); }
        }

        public double TrimLow
        {
            set { _trimLow = value / 200 - 0.1; }
        }

        public void Update(double decibels)
        {
            // ------------------------------------------------------ test
            //decibels = 0;
            //decibels = 30;
            //decibels = 40;
            //decibels = -99;
            // ------------------------------------------------------ Trim LOW and HI
            double veff = Math.Pow(10, decibels / 20);
            veff += _trimLow;
            veff *= _trimHi;
            if (veff < 0.0005) veff = 0.0005;
            double db = 20 * Math.Log10(veff);
            // ------------------------------------------------------ 
            if (db < _minDecibels) db = _minDecibels;
            if (db > _maxDecibels) db = _maxDecibels;
            double a = krad * (_zeroAngle + _dbAngle * db);
            _oldAngle += (a - _oldAngle) * _speed;
            float _sin = (float)Math.Sin(_oldAngle);
            float _cos = (float)Math.Cos(_oldAngle);
            // ------------------------------------------------------ draw
            g.Clear(BackColor);
            g.DrawImage(_backBitmap, 0, 0);
            g.DrawLine(_armPen, x0, y0,
                        x0 + _sin * _lineLenPixels,
                        y0 - _cos * _lineLenPixels);
            g.FillRectangle(_transparentBrush, 0, h - 12, w, 12);
            // ------------------------------------------------------ uV text
            veff *= 55;
            if (veff > peakVeff || sw.ElapsedMilliseconds > 300)
            {
                peakVeff = veff;
                sw.Reset();
                sw.Start();
            }
            if (peakVeff >= 1000) g.DrawString((peakVeff * 0.001).ToString("0.0 mV"), Font, _textBrush, 2, 2);
            else if (peakVeff >= 10) g.DrawString(peakVeff.ToString("0 uV"), Font, _textBrush, 2, 2);
            else g.DrawString(peakVeff.ToString("0.0 uV"), Font, _textBrush, 2, 2);
            //
            Invalidate();
        }
    }
}