Imports System.ComponentModel
Imports System.Drawing
Imports System.Windows.Forms
Imports System.ComponentModel.Design

' --------------------------------------------------------------------
'  To avoid problems produced by the 100%, 125% and 150% DPI settings
'  this project must be compiled with the following flag disabled:
'   "Compatibility" / "Disable Display Scaling On High DPI Settings"
'   (in the file properties of "xxx.exe" and "xxx.vshost.exe")
' --------------------------------------------------------------------

Namespace MyPanels
	Public Enum PanelStateOptions
		Collapsed
		Expanded
	End Enum

	<DesignTimeVisible(True)> _
	<Category("Containers")> _
	<Description("Visual Studio like Panel")> _
	<Designer("System.Windows.Forms.Design.ParentControlDesigner, System.Design", GetType(IDesigner))> _
	Public Partial Class MyPanel
		Inherits UserControl
		Private _expandedHeight As Integer
		Private _panelState As PanelStateOptions = PanelStateOptions.Expanded
		Private _isCollapsed As Boolean = False
		Private _fitToParent As Boolean = False
		Private _nextPanel As MyPanel

		Public Delegate Sub DelegateStateChanged()
		<Category("Panels")> _
		Public Event StateChanged As DelegateStateChanged

		<Description("Gets or sets the value for user control height when it is expanded")> _
		<DisplayName("Expanded Height")> _
		<Category("Panels")> _
		<DefaultValueAttribute(0)> _
		Public Property ExpandedHeight() As Integer
			Get
				Return _expandedHeight
			End Get
			Set
				If value > 0 Then
					If Me.DesignMode Then
						If _panelState = PanelStateOptions.Expanded Then
							'Setting Expanded Height is only allowed when user control is expanded
							Me.SetBounds(Me.Location.X, Me.Location.Y, Me.Size.Width, titlePanel.Height + value)
						End If
					Else
						_expandedHeight = value
					End If
				End If
			End Set
		End Property

		<Description("Gets or sets panel title")> _
		<DisplayName("Panel Title")> _
		<Category("Panels")> _
		Public Property PanelTitle() As String
			Get
				Return lblPanelTitle.Text
			End Get
			Set
				lblPanelTitle.Text = value
			End Set
		End Property

		<DefaultValue(GetType(PanelStateOptions), "Expanded")> _
		<Description("Gets or sets current panel state")> _
		<DisplayName("Panel State")> _
		<Category("Panels")> _
		Public Property PanelState() As PanelStateOptions
			Get
				Return _panelState
			End Get
			Set
				_panelState = value
				_isCollapsed = (_panelState <> PanelStateOptions.Collapsed)
				ToggleState(Nothing, Nothing)
			End Set
		End Property

		<Category("Panels")> _
		<DesignOnly(True)> _
		<DefaultValue(False)> _
		<Description("If True, fits the panel to match the parent width")> _
		Public Property FitToParent() As Boolean
			Get
				Return _fitToParent
			End Get
			Set
				_fitToParent = value

				If _fitToParent Then
					If Me.Parent IsNot Nothing Then
						Me.Location = New Point(0, Me.Location.Y)
						Me.Size = New Size(Me.Parent.Size.Width, Me.Size.Height)
						Me.Anchor = AnchorStyles.Top Or AnchorStyles.Left Or AnchorStyles.Right
					End If
				Else
					Me.Anchor = AnchorStyles.Top Or AnchorStyles.Left
				End If
			End Set
		End Property

		<Category("Panels")> _
		<Description("Gets or sets the panel to be located beneath this panel")> _
		Public Property NextPanel() As MyPanel
			Get
				Return _nextPanel
			End Get
			Set
				_nextPanel = value
				MoveNextPanel()
			End Set
		End Property

		Public Sub New()
			InitializeComponent()
			AddHandler Me.Load, New EventHandler(AddressOf Panels_Load)
			AddHandler Me.SizeChanged, New EventHandler(AddressOf Panels_SizeChanged)
			AddHandler Me.LocationChanged, New EventHandler(AddressOf Panels_LocationChanged)
		End Sub

		Private Sub Panels_MouseEnter(sender As Object, e As EventArgs)
			'this.Focus();
		End Sub

		Private Sub Panels_Load(sender As Object, e As EventArgs)
			_isCollapsed = (_panelState = PanelStateOptions.Collapsed)
		End Sub

		Private Sub Panels_SizeChanged(sender As Object, e As EventArgs)
			If Me.DesignMode Then
				If _panelState = PanelStateOptions.Expanded Then
					_expandedHeight = Me.Height - titlePanel.Height
				Else
					' Final user can only resize the user control when it is expanded
					Me.SetBounds(Me.Location.X, Me.Location.Y, Me.Size.Width, titlePanel.Height)
				End If

				If Me.Parent IsNot Nothing Then
					If Me.Parent.Size.Width <> Me.Size.Width Then
						FitToParent = False
					End If
				End If
			End If
			MoveNextPanel()
		End Sub

		Private Sub Panels_LocationChanged(sender As Object, e As EventArgs)
			If Me.DesignMode Then
				If Me.Location.X > 0 Then
					FitToParent = False
				End If
			End If
			MoveNextPanel()
		End Sub

		Private Sub ToggleState(sender As Object, e As EventArgs)
			If _isCollapsed Then
				' --------------------------------------------------------------------
				'  Here the _expandedHeight is run time recalculated
				'  to overcome problems produced by the 125% or 150% DPI settings
				'  end the presence or not of the flag in the EXE property:
				'    "Compatibility" / "Disable Display Scaling On High DPI Settings" 
				' --------------------------------------------------------------------
				Dim h As Integer = 0
				For i As Integer = 0 To Me.Controls.Count - 1
					Dim b As Integer = Me.Controls(i).Bottom
					If b > h Then
						h = b
					End If
				Next
				_expandedHeight = h - titlePanel.Height + 5

				' ----------------------------------- Expand the user control
				Me.SetBounds(Me.Location.X, Me.Location.Y, Me.Size.Width, titlePanel.Height + _expandedHeight)

				' ----------------------------------- Scroll the control completely visibe 
				Dim container As Panel = TryCast(Me.Parent.Parent, Panel)
				container.ScrollControlIntoView(Me)
			Else
				' ----------------------------------- Collapse the user control
				Me.SetBounds(Me.Location.X, Me.Location.Y, Me.Size.Width, titlePanel.Height)
			End If

			_isCollapsed = Not _isCollapsed

			' Setting content control current state and toggling image
			If _isCollapsed Then
				_panelState = PanelStateOptions.Collapsed
			Else
				_panelState = PanelStateOptions.Expanded
			End If

			If Not Me.DesignMode Then
				RaiseEvent StateChanged()
			End If
		End Sub

		Private Sub MoveNextPanel()
			If _nextPanel IsNot Nothing Then
				_nextPanel.Location = New Point(_nextPanel.Location.X, Me.Location.Y + Me.Size.Height)
			End If
		End Sub
	End Class
End Namespace
